@@ -1,8 +1,29 @@
-/* -*- mode: C; c-basic-offset: 4 -*- */
+/* -*- mode: C; c-basic-offset: 4 -*-
+ * pyorbit - a Python language mapping for the ORBit2 CORBA ORB
+ * Copyright (C) 2002  James Henstridge <james@daa.com.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 #include "pyorbit-private.h"
+#include <orbit/poa/poa-types.h>
+
+PortableServer_POA _pyorbit_poa;
 
 static PyObject *
 pyorbit_load_typelib(PyObject *self, PyObject *args)
@@ -20,13 +41,22 @@ pyorbit_load_typelib(PyObject *self, PyObject *args)
     }
     ifaces = ORBit_small_get_iinterfaces(typelib);
     for (i = 0; i < ifaces->_length; i++) {
+	if (ifaces->_buffer[i].tc->kind == CORBA_tk_null)
+	    g_warning("typelib %s is possibly broken: tc->kind == tk_null",
+		      typelib);
+
 	pyorbit_generate_iinterface_stubs(&ifaces->_buffer[i]);
+	_pyorbit_register_skel(&ifaces->_buffer[i]);
     }
     CORBA_free(ifaces);
     types = ORBit_small_get_types(typelib);
     for (i = 0; i < types->_length; i++) {
 	CORBA_TypeCode tc = types->_buffer[i];
 
+	if (tc->kind == CORBA_tk_null ||
+	    (tc->kind==CORBA_tk_alias && tc->subtypes[0]->kind==CORBA_tk_null))
+	    g_warning("typelib %s is possibly broken: tc->kind == tk_null",
+		      typelib);
 	pyorbit_generate_typecode_stubs(tc);
     }
     CORBA_free(types);
@@ -75,8 +105,13 @@ pyorbit_corba_orb_init(PyObject *self, PyObject *args)
     CORBA_exception_init(&ev);
     orb = CORBA_ORB_init(&argc, argv, orb_id, &ev);
     g_free(argv);
+    _pyorbit_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
     if (pyorbit_check_ex(&ev))
 	return NULL;
+    PortableServer_POAManager_activate (PortableServer_POA__get_the_POAManager (_pyorbit_poa, &ev), &ev);
+    if (pyorbit_check_ex(&ev))
+	return NULL;
+
     pyorb = pycorba_orb_new(orb);
     CORBA_Object_duplicate((CORBA_Object)orb, NULL);
     return pyorb;
@@ -224,17 +259,41 @@ register_corba_types(void)
     pyorbit_generate_typecode_stubs(TC_CORBA_ORB_InvalidName);
 }
 
+static PyMethodDef portableserver_functions[] = {
+    { NULL, NULL, 0 }
+};
+
+static struct _PyORBit_APIStruct api = {
+    &PyCORBA_Object_Type,
+    &PyCORBA_ORB_Type,
+    &PyCORBA_TypeCode_Type,
+    &PyCORBA_Any_Type,
+    &PyPortableServer_POA_Type,
+    &PyPortableServer_POAManager_Type,
+
+    pycorba_object_new,
+    pycorba_orb_new,
+    pycorba_typecode_new,
+    pycorba_any_new,
+    pyorbit_poa_new,
+    pyorbit_poamanager_new,
+
+    pyorbit_check_ex,
+    pyorbit_marshal_any,
+    pyorbit_demarshal_any
+};
+
 void initORBit(void);
 
 DL_EXPORT(void)
 initORBit(void)
 {
-    PyObject *mod, *corbamod;
+    PyObject *mod, *corbamod, *psmod;
 
 #define INIT_TYPE(tp) G_STMT_START{ \
-    tp.ob_type = &PyType_Type; \
-    tp.tp_alloc = PyType_GenericAlloc; \
-    tp.tp_new = PyType_GenericNew; \
+    if (!tp.ob_type)  tp.ob_type  = &PyType_Type; \
+    if (!tp.tp_alloc) tp.tp_alloc = PyType_GenericAlloc; \
+    if (!tp.tp_new)   tp.tp_new   = PyType_GenericNew; \
     if (PyType_Ready(&tp) < 0) \
         return; }G_STMT_END
 
@@ -247,6 +306,12 @@ initORBit(void)
     INIT_TYPE(PyCORBA_Struct_Type);
     INIT_TYPE(PyCORBA_Union_Type);
     INIT_TYPE(PyCORBA_UnionMember_Type);
+    PyCORBA_Enum_Type.tp_base = &PyInt_Type;
+    INIT_TYPE(PyCORBA_Enum_Type);
+    INIT_TYPE(PyCORBA_fixed_Type);
+    INIT_TYPE(PyPortableServer_Servant_Type);
+    INIT_TYPE(PyPortableServer_POA_Type);
+    INIT_TYPE(PyPortableServer_POAManager_Type);
 
 #undef INIT_TYPE
 
@@ -258,6 +323,9 @@ initORBit(void)
 				     orbit_minor_version,
 				     orbit_micro_version));
 
+    PyModule_AddObject(mod, "_PyORBit_API",
+		       PyCObject_FromVoidPtr(&api, NULL));
+
     corbamod = Py_InitModule("CORBA", corba_functions);
 
     PyModule_AddObject(corbamod, "TypeCode",
@@ -268,6 +336,8 @@ initORBit(void)
 		       (PyObject *)&PyCORBA_ORB_Type);
     PyModule_AddObject(corbamod, "Any",
 		       (PyObject *)&PyCORBA_Any_Type);
+    PyModule_AddObject(corbamod, "fixed",
+		       (PyObject *)&PyCORBA_fixed_Type);
 
     PyModule_AddObject(corbamod, "TRUE",  Py_True);
     PyModule_AddObject(corbamod, "FALSE", Py_False);
@@ -275,4 +345,12 @@ initORBit(void)
     pyorbit_register_exceptions(corbamod);
 
     register_corba_types();
+
+    psmod = Py_InitModule("PortableServer", portableserver_functions);
+    PyModule_AddObject(psmod, "POA",
+		       (PyObject *)&PyPortableServer_POA_Type);
+    PyModule_AddObject(psmod, "POAManager",
+		       (PyObject *)&PyPortableServer_POAManager_Type);
+    PyModule_AddObject(psmod, "Servant",
+		       (PyObject *)&PyPortableServer_Servant_Type);
 }
