@@ -1,9 +1,29 @@
-/* -*- mode: C; c-basic-offset: 4 -*- */
+/* -*- mode: C; c-basic-offset: 4 -*-
+ * pyorbit - a Python language mapping for the ORBit2 CORBA ORB
+ * Copyright (C) 2002  James Henstridge <james@daa.com.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 #include "pyorbit-private.h"
 
+size_t  ORBit_gather_alloc_info (CORBA_TypeCode tc);
+
 #define alignval(val,align)    *val = ALIGN_ADDRESS(*val, align)
 #define getval(val,type)       (*(type *)*val)
 #define advanceptr(val,offset) *val = ((guchar *)*val) + offset
@@ -214,6 +234,11 @@ marshal_value(CORBA_TypeCode tc, gconstpointer *val, PyObject *value)
 	    getval(val, CORBA_string) = CORBA_string_dup(PyString_AsString(value));
 	    advanceptr(val, sizeof(CORBA_char *));
 	    ret = TRUE;
+	} else if (value == Py_None) {
+	    alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
+	    getval(val, CORBA_string) = NULL;
+	    advanceptr(val, sizeof(CORBA_char *));
+	    ret = TRUE;
 	}
 	break;
     case CORBA_tk_sequence: 
@@ -311,6 +336,11 @@ marshal_value(CORBA_TypeCode tc, gconstpointer *val, PyObject *value)
 	    }
 	    advanceptr(val, sizeof(CORBA_wchar *));
 	    ret = TRUE;
+	} else if (value == Py_None) {
+	    alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
+	    getval(val, CORBA_wstring) = NULL;
+	    advanceptr(val, sizeof(CORBA_wchar *));
+	    ret = TRUE;
 	}
 	break;
     default:
@@ -362,7 +392,6 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	advanceptr(val, sizeof(CORBA_unsigned_short));
 	break;
     case CORBA_tk_ulong:
-    case CORBA_tk_enum:
 	alignval(val, ORBIT_ALIGNOF_CORBA_LONG);
 	ret = PyLong_FromUnsignedLong(getval(val, CORBA_unsigned_long));
 	advanceptr(val, sizeof(CORBA_unsigned_long));
@@ -410,8 +439,7 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	break;
     case CORBA_tk_objref:
 	alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
-	ret = pycorba_object_new(
-		CORBA_Object_duplicate(getval(val, CORBA_Object), NULL));
+	ret = pycorba_object_new(getval(val, CORBA_Object));
 	advanceptr(val, sizeof(CORBA_Object));
 	break;
     case CORBA_tk_struct:
@@ -426,9 +454,15 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	    if (tc->kind == CORBA_tk_struct)
 		stub = (PyObject *)&PyBaseObject_Type;
 	    else
-		stub = PyExc_Exception;
+		stub = pyorbit_exception;
 	}
 	instance = PyObject_CallFunction(stub, "()");
+	if (stub == pyorbit_exception) {
+	    PyObject *pytc = pycorba_typecode_new(tc);
+
+	    PyObject_SetAttrString(instance, "__typecode__", pytc);
+	    Py_DECREF(pytc);
+	}
 	for (i = 0; i < tc->sub_parts; i++) {
 	    PyObject *item;
 	    gchar *pyname;
@@ -493,17 +527,32 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	advanceptr(val, sz);
 	break;
     }
-    case CORBA_tk_string:
-	alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
-	ret = PyString_FromString(getval(val, CORBA_string));
-	advanceptr(val, sizeof(CORBA_string));
+    case CORBA_tk_enum:
+	alignval(val, ORBIT_ALIGNOF_CORBA_LONG);
+	ret = pycorba_enum_from_long(tc, getval(val, CORBA_unsigned_long));
+	advanceptr(val, sizeof(CORBA_unsigned_long));
 	break;
+    case CORBA_tk_string: {
+	CORBA_string str;
+
+	alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
+	str = getval(val, CORBA_string);
+	advanceptr(val, sizeof(CORBA_string));
+	if (str) {
+	    ret = PyString_FromString(str);
+	} else {
+	    Py_INCREF(Py_None);
+	    ret = Py_None;
+	}
+	break;
+    }
     case CORBA_tk_sequence: {
 	const CORBA_sequence_CORBA_octet *sval;
 	gconstpointer seqval;
 
 	alignval(val, ORBIT_ALIGNOF_CORBA_SEQ);
 	sval = (CORBA_sequence_CORBA_octet *)*val;
+	advanceptr(val, sizeof(CORBA_sequence_CORBA_octet));
 	switch (tc->subtypes[0]->kind) {
 	case CORBA_tk_char:
 	case CORBA_tk_octet:
@@ -519,7 +568,7 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	    for (i = 0; i < sval->_length; i++) {
 		PyObject *item;
 
-		alignval(val, align);
+		alignval(&seqval, align);
 		item = demarshal_value(tc->subtypes[0], &seqval);
 		if (!item) {
 		    Py_DECREF(tuple);
@@ -577,16 +626,24 @@ demarshal_value(CORBA_TypeCode tc, gconstpointer *val)
 	break;
     }
     case CORBA_tk_wstring: {
-	CORBA_wstring wstr = getval(val, CORBA_wstring);
-	gint length = CORBA_wstring_len(wstr), i;
-	Py_UNICODE *ustr;
+	CORBA_wstring wstr;
 
 	alignval(val, ORBIT_ALIGNOF_CORBA_POINTER);
-	ustr = g_new(Py_UNICODE, length);
-	for (i = 0; i < length; i++) ustr[i] = wstr[i];
-	ret = PyUnicode_FromUnicode(ustr, length);
-	g_free(ustr);
+	wstr = getval(val, CORBA_wstring);
 	advanceptr(val, sizeof(CORBA_wstring));
+
+	if (wstr) {
+	    gint i, length = CORBA_wstring_len(wstr);
+	    Py_UNICODE *ustr;
+
+	    ustr = g_new(Py_UNICODE, length);
+	    for (i = 0; i < length; i++) ustr[i] = wstr[i];
+	    ret = PyUnicode_FromUnicode(ustr, length);
+	    g_free(ustr);
+	} else {
+	    Py_INCREF(Py_None);
+	    ret = Py_None;
+	}
 	break;
     }
     default:
