@@ -1,4 +1,22 @@
-/* -*- mode: C; c-basic-offset: 4 -*- */
+/* -*- mode: C; c-basic-offset: 4 -*-
+ * pyorbit - a Python language mapping for the ORBit2 CORBA ORB
+ * Copyright (C) 2002  James Henstridge <james@daa.com.au>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -108,9 +126,7 @@ pycorba_typecode_repr(PyCORBA_TypeCode *self)
 static PyObject *
 pycorba_typecode_get_kind(PyCORBA_TypeCode *self, void *closure)
 {
-    if (self->tc->kind < G_MAXLONG)
-	return PyInt_FromLong(self->tc->kind);
-    return PyLong_FromUnsignedLong(self->tc->kind);
+    return pycorba_enum_from_long(TC_CORBA_TCKind, self->tc->kind);
 }
 
 static PyObject *
@@ -337,7 +353,7 @@ PyTypeObject PyCORBA_TypeCode_Type = {
     (getattrofunc)0,                    /* tp_getattro */
     (setattrofunc)0,                    /* tp_setattro */
     0,                                  /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,   /* tp_flags */
+    Py_TPFLAGS_DEFAULT,                 /* tp_flags */
     NULL, /* Documentation string */
     (traverseproc)0,                    /* tp_traverse */
     (inquiry)0,                         /* tp_clear */
